#!/usr/bin/env python
'''
This file contains Python command example for "What just happened" feature.
This example demonstrates the following :
a. configuration of trap group for WJH feature.
b. Retrieval of packets from the Queue associated with WJH
Python commands syntax is very similar to the Switch SDK APIs.
You can learn more about each command and its parameters by reading the SwitchX API Reference Manual.

The example below demonstrates the configuration of different host interface trap attributes.
This example is supported on Spectrum devices.
'''
import sys
import errno
import sys
import colorsys
import cmd
import argparse

sys.path.append('/usr/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/site-packages/python_sdk_api/')
sys.path.append('/usr/lib/python2.7/dist-packages/python_sdk_api/')
sys.path.append('/usr/local/lib/python2.7/dist-packages/python_sdk_api/')
from sx_api import *

dest_port_type_dict = {0: 'INVALID', 1: 'MULTI_PORT', 2: 'NETWORK_PORT', 4: 'LAG_PORT'}

print "[+] opening sdk"
rc, handle = sx_api_open(None)
print ("sx_api_open handle:0x%x , rc %d " % (handle, rc))
if (rc != SX_STATUS_SUCCESS):
    print "Failed to open api handle.\nPlease check that SDK is running."
    sys.exit(errno.EACCES)

######################################################
#    defines
######################################################
SPECTRUM_SWID = 0
WAIT_FOR_KEY = 0
DO_INF_LOOP = 0

parser = argparse.ArgumentParser(description='Port Counter dump utility')
parser.add_argument('--wait_enable', default='false', choices=['true', 'false'], help='Wait for key after WJH operations')
parser.add_argument('--infinite_read', default='false', choices=['true', 'false'], help='Read the packets arrived to WJH in infinite loop')
args = parser.parse_args()
if args.wait_enable == 'true':
    WAIT_FOR_KEY = 1
if args.infinite_read == 'true':
    DO_INF_LOOP = 1
print ("DO_INF_LOOP %s , WAIT_FOR_KEY: %s " % (DO_INF_LOOP, WAIT_FOR_KEY))


def local_raw_input(msg):
    if WAIT_FOR_KEY != 0:
        raw_input(msg)


def print_recv_info(recv_info_p, pkt_size_p, pkt):

    if recv_info_p.is_lag:
        print ("Source LAG 0x%x" % recv_info_p.source_lag_port)
    print ("Source log port 0x%x" % recv_info_p.source_log_port)

    print ("Recv trap id: %d" % recv_info_p.trap_id)

    print ("Destination port type : %s" % dest_port_type_dict[recv_info_p.dest_port_type])
    if recv_info_p.dest_port_type == SX_RECEIVE_DEST_PORT_NETWORK_PORT_E:
        print ("Destination log port 0x%x" % recv_info_p.dest_log_port)
    if recv_info_p.dest_port_type == SX_RECEIVE_DEST_PORT_LAG_PORT_E:
        print ("Destination LAG 0x%x" % recv_info_p.dest_lag_port)
        print ("Destination log port 0x%x" % recv_info_p.dest_log_port)

    pkt_size = pkt_size_p
    print ("Packet size: %d" % pkt_size)
    print ("Packet content:")
    if (pkt_size > 0):
        for iii in range(0, pkt_size):
            if (0 == iii % 8):
                print("%02x %02x %02x %02x %02x %02x %02x %02x " %
                      (uint8_t_arr_getitem(pkt, iii),
                       uint8_t_arr_getitem(pkt, iii + 1),
                       uint8_t_arr_getitem(pkt, iii + 2),
                       uint8_t_arr_getitem(pkt, iii + 3),
                       uint8_t_arr_getitem(pkt, iii + 4),
                       uint8_t_arr_getitem(pkt, iii + 5),
                       uint8_t_arr_getitem(pkt, iii + 6),
                       uint8_t_arr_getitem(pkt, iii + 7)))

        print("")


def sx_recv_multi_count(fd_p):

    pkt_info_arr_len_p = new_uint32_t_p()
    uint32_t_p_assign(pkt_info_arr_len_p, 0)

    print ("[recv] Count WJH fd")
    rc = sx_lib_host_ifc_recv_list(fd_p, None, pkt_info_arr_len_p)
    if rc != 0:
        print ("exit with error, rc %d" % rc)
        exit(rc)
    pkt_info_arr_len = uint32_t_p_value(pkt_info_arr_len_p)

    print ("[recv] %d Packet count in queue :" % (pkt_info_arr_len))


def sx_recv_multi_flush(fd_p):

    print ("[recv] Flush WJH fd")
    rc = sx_lib_host_ifc_recv_list(fd_p, None, None)
    if rc != 0:
        print ("exit with error, rc %d" % rc)
        exit(rc)
    print ("[recv] Flush of WJH fd is succeeded,")


def sx_recv_multi(fd_p):

    pkt_arr = []
    # recv parameters
    pkt_size = 2000
    pkt_size_p = new_uint32_t_p()
    uint32_t_p_assign(pkt_size_p, pkt_size)
    pkt0 = new_uint8_t_arr(pkt_size)
    pkt1 = new_uint8_t_arr(pkt_size)
    pkt2 = new_uint8_t_arr(pkt_size)
    #pkt_arr[0] = new_uint8_t_arr(pkt_size)
    pkt_arr.append(pkt0)
    pkt_arr.append(pkt1)
    pkt_arr.append(pkt2)

    recv_info_p = new_sx_receive_info_t_p()
    pkt_info_arr = new_sx_packet_info_t_arr(10)
    pkt_info = sx_packet_info_t()
    pkt_info.packet_p = pkt0
    pkt_info.packet_size = pkt_size
    sx_packet_info_t_arr_setitem(pkt_info_arr, 0, pkt_info)
    pkt_info = sx_packet_info_t()
    pkt_info.packet_p = pkt1
    pkt_info.packet_size = pkt_size
    sx_packet_info_t_arr_setitem(pkt_info_arr, 1, pkt_info)
    pkt_info = sx_packet_info_t()
    pkt_info.packet_p = pkt2
    pkt_info.packet_size = pkt_size
    sx_packet_info_t_arr_setitem(pkt_info_arr, 2, pkt_info)
    pkt_info_arr_len_p = new_uint32_t_p()
    uint32_t_p_assign(pkt_info_arr_len_p, 3)

    print ("[recv] recv_multi on fd")
    rc = sx_lib_host_ifc_recv_list(fd_p, pkt_info_arr, pkt_info_arr_len_p)
    if rc != 0:
        print ("exit with error, rc %d" % rc)
        exit(rc)
    pkt_info_arr_len = uint32_t_p_value(pkt_info_arr_len_p)

    print ("[recv] %d Packet recved. Recv  info:" % (pkt_info_arr_len))
    pkt_info = sx_packet_info_t()
    recv_info_p = sx_receive_info_t()

    for i in range(pkt_info_arr_len):
        print ("=" * 50)
        print ("Read packet from index %d " % (i))
        pkt_info = sx_packet_info_t_arr_getitem(pkt_info_arr, i)
        recv_info_p = pkt_info.receive_info
        print_recv_info(recv_info_p, pkt_info.packet_size, pkt_arr[i])

    print ("=" * 50)
    return


def sx_recv(fd_p):
    # recv parameters
    pkt_size = 2000
    pkt_size_p = new_uint32_t_p()
    uint32_t_p_assign(pkt_size_p, pkt_size)
    pkt = new_uint8_t_arr(pkt_size)
    recv_info_p = new_sx_receive_info_t_p()

    print ("[recv] recv on fd")
    rc = sx_lib_host_ifc_recv(fd_p, pkt, pkt_size_p, recv_info_p)
    if rc != 0:
        print ("exit with error, rc %d" % rc)
        exit(rc)

    print ("[recv] Packet recv info:")
    print ("=" * 50)
    if recv_info_p.is_lag:
        print ("Source LAG 0x%x" % recv_info_p.source_lag_port)
    print ("Source log port 0x%x" % recv_info_p.source_log_port)

    print ("Recv trap id: %d" % recv_info_p.trap_id)

    print ("Destination port type : %s" % dest_port_type_dict[recv_info_p.dest_port_type])
    if recv_info_p.dest_port_type == SX_RECEIVE_DEST_PORT_NETWORK_PORT_E:
        print ("Destination log port 0x%x" % recv_info_p.dest_log_port)
    if recv_info_p.dest_port_type == SX_RECEIVE_DEST_PORT_LAG_PORT_E:
        print ("Destination LAG 0x%x" % recv_info_p.dest_lag_port)
        print ("Destination log port 0x%x" % recv_info_p.dest_log_port)

    print ("ACL user id: %d" % recv_info_p.acl_user_id)

    pkt_size = uint32_t_p_value(pkt_size_p)
    print ("Packet size: %d" % pkt_size)

    print ("Packet content:")

    if (pkt_size > 0):
        for iii in range(0, pkt_size):
            if (0 == iii % 8):
                # print("")
                print("%02x %02x %02x %02x %02x %02x %02x %02x " %
                      F(uint8_t_arr_getitem(pkt, iii),
                        uint8_t_arr_getitem(pkt, iii + 1),
                        uint8_t_arr_getitem(pkt, iii + 2),
                        uint8_t_arr_getitem(pkt, iii + 3),
                        uint8_t_arr_getitem(pkt, iii + 4),
                        uint8_t_arr_getitem(pkt, iii + 5),
                        uint8_t_arr_getitem(pkt, iii + 6),
                        uint8_t_arr_getitem(pkt, iii + 7)))

        print("")
    print ("=" * 50)

    print ("[recv] recv done")


""" ############################################################################################ """


def host_ifc_trap_id_register(cmd, trap_id, user_channel_p):
    """ HOST IFC TRAP ID REGISTER/UNREGISTER """
    if cmd == SX_ACCESS_CMD_REGISTER:
        print ("--------------- HOST IFC TRAP ID REGISTER------------------------------")
    else:
        print ("--------------- HOST IFC TRAP ID DEREGISTER------------------------------")

    rc = sx_api_host_ifc_trap_id_register_set(handle, cmd, SPECTRUM_SWID, trap_id, user_channel_p)
    print("sx_api_host_ifc_trap_id_register_set [rc = %d]" % (rc))

    if rc != SX_STATUS_SUCCESS:
        sys.exit(errno.EACCES)


""" ############################################################################################ """
""" HOST IFC OPEN """
print ("--------------- HOST IFC OPEN------------------------------")

fd_p = new_sx_fd_t_p()

rc = sx_api_host_ifc_open(handle, fd_p)
print ("sx_api_host_ifc_open, rc=%d] " % (rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
fd = sx_fd_t_p_value(fd_p)
print("fd = %d" % (fd.fd))


""" ############################################################################################ """
# sx_api_host_ifc_trap_group_set 0 1 {1;SX_TRUNCATE_MODE_DISABLE;0;SX_CONTROL_TYPE_DEFAULT}
""" HOST IFC TRAP GROUP SET """
print ("--------------- HOST IFC TRAP GROUP SET ------------------------------")

trap_group = 1
trap_group_attr_p = new_sx_trap_group_attributes_t_p()
trap_group_attr = sx_trap_group_attributes_t()
# trap_group_attr.prio = 1
trap_group_attr.prio = 5
trap_group_attr.truncate_mode = SX_TRUNCATE_MODE_DISABLE
trap_group_attr.truncate_size = 0
trap_group_attr.control_type = SX_CONTROL_TYPE_DEFAULT
trap_group_attr.is_monitor = 1
trap_group_attr.monitor_fd = fd

sx_trap_group_attributes_t_p_assign(trap_group_attr_p, trap_group_attr)

rc = sx_api_host_ifc_trap_group_set(handle, SPECTRUM_SWID, trap_group, trap_group_attr_p)
print("sx_api_host_ifc_trap_group_set [rc = %d]" % (rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)

trap_group_attr = sx_trap_group_attributes_t_p_value(trap_group_attr_p)
print("trap_group =%d " % (trap_group))
print("--- trap_group attributes ---")
print("prio  =%d " % (trap_group_attr.prio))
print("truncate mode  =%d " % (trap_group_attr.truncate_mode))
print("truncate size  =%d " % (trap_group_attr.truncate_size))
print("control type  =%d " % (trap_group_attr.control_type))

""" ############################################################################################ """

# sx_api_host_ifc_trap_id_set 0 SX_TRAP_ID_ETH_L2_STP 1 SX_TRAP_ACTION_TRAP_2_CPU
""" HOST IFC TRAP ID SET """
print ("--------------- HOST IFC TRAP ID SET------------------------------")

trap_id = SX_TRAP_ID_ARP_REQUEST
trap_action = SX_TRAP_ACTION_TRAP_2_CPU

rc = sx_api_host_ifc_trap_id_set(handle, SPECTRUM_SWID, trap_id, trap_group, trap_action)
print("sx_api_host_ifc_trap_id_set [rc=%d]" % (rc))

if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
print("trap_id =%d, trap_group=%d, trap_action =%d, rc=%d" % (trap_id, trap_group, trap_action, rc))


local_raw_input("Press key to read the packets from monitor ...")
# sx_recv(fd_p)
while True:
    sx_recv_multi(fd_p)
    local_raw_input("Press Enter key to read count 1 ...")
    sx_recv_multi_count(fd_p)
    local_raw_input("Press Enter key to read count 2 ...")
    sx_recv_multi_count(fd_p)
    local_raw_input("Press Enter key to read count 3 ...")
    sx_recv_multi_count(fd_p)
    local_raw_input("Press Enter key to flush ...")
    sx_recv_multi_flush(fd_p)
    local_raw_input("Press Enter key to read count after flush ...")
    sx_recv_multi_count(fd_p)
    local_raw_input("Press Enter key to continue to recv packets ...")
    if DO_INF_LOOP == 0:
        break


""" ############################################################################################ """
""" HOST IFC CLOSE """
print ("--------------- HOST IFC CLOSE------------------------------")

rc = sx_api_host_ifc_close(handle, fd_p)
print ("sx_api_host_ifc_close, rc=%d] " % (rc))
if rc != SX_STATUS_SUCCESS:
    sys.exit(errno.EACCES)
fd = sx_fd_t_p_value(fd_p)
print("fd = %d" % (fd.fd))


""" ############################################################################################ """
sx_api_close(handle)
""" ############################################################################################ """
